.\" For license terms, see the file COPYING in this directory.
.TH dcpc 3
.SH NAME
dcpc \- client library for Distributed Computing Protocol
.SH SYNOPSIS
\fB#include <dcpc.h>\fR

int dcp_errno;

int dcp_open(const char *server, int portnum)

int dcp_hello(const char *userid)

int dcp_select(const char *project, const char *clientid, char *secret)

struct dcpc_entry *dcp_view_key(const char *key)

int dcp_view_range(int min, int max, int *count,
                   void (*hook)(const char *, struct dcpc_entry *))

struct dcpc_entry *dcp_request_key(const char *key)

int dcp_request_range(int min, int max, int *count,
                   void (*hook)(const char *, struct dcpc_entry *))

int dcp_report_key(const char *key, const char *data)

struct dcpc_entry *dcp_release_key(const char *key)

int dcp_release_range(int min, int max, int *count,
                   void (*hook)(const char *, struct dcpc_entry *))

int dcp_close(void);

char *dcp_error(int err)

.SH DESCRIPTION
The 
.I dcpc
library assists programs in speaking DCP (Distributed Computing
Protocol).  It is intended for the use of programmers writing
DCP client daemons specialized for particular DCP projects, such
as the Great Internet Mersenne Prime Search.
.PP
The uses and specifics of DCP are described in the 
.I DCP Protocol Specification
document included with this distribution.
.PP
The library encapsulates the DCP protocol in a simple application
programming interface, translating data to be shipped to and from the
server in appropriate ways.  
.PP
Link this library with the linker flag -ldcpc.
.PP
Integer-valued entry points return the manifest constant DCP_OK on
success, DCP_ERR on error.  Routines returning a report structure
pointer return NULL on error.  Additionally, the global variable
\fBdcp_errno\fR is set with an appropriate error code from \fBdcp_error.h\fR.  
.SH ENTRY POINTS
.PP
The \fBdcp_open\fR function attempts to initiate a TCP/IP connection
with a DCP server.  You must give it a server name string, a port
number.
.PP
The \fIdcp_hello\fR function identifies you to the server.  You must
give it a user ID, which should be your canonical Internet @-address,
.PP
The \fIdcp_select\fR function attempts to authenticate you for access
to a given project database.  You must give it a project ID and a client ID string
identifying your client program type and release level.
.PP
If you specify a non-NULL \fBsecret\fR argument, it is used in an
attempt to authenticate you to the server.  On successful return,
the buffer to which \fBsecret\fR points will be overwritten with
a new secret, to be used for your next authentication to that
project on that server.
.PP
All other functions assume that \fBdcp_open\fR has returned successfully.
.PP
The \fBdcp_view_key\fR function allows you to retrieve the data associated
with a single (string) key. 
.PP
The \fBdcp_view_range\fR function allows you to retrieve the data
associated with a range of numeric keys.  The actual number of valid
keys in the range is first returned in *count.  Then the hook function is
called once for each valid key.  
.PP
The \fBdcp_request_key\fR function allows you to reserve a single
(string) key, provided you have an authenticated connection.  If the
key argument is NULL, one will be generated by the server and passed back.
The returned report structure represent the data associated with the
key \fIfBafter\fR the reservation attempt.
.PP
The \fBdcp_request_range\fR function allows you to try to reserve a
range of numeric keys, provided you have an authenticated connection.
The actual number of valid keys in the range is first returned in
*count.  Then the hook function is called once for each valid key.
The argument passed to the hook represents each valid key's data
\fBafter\fR the attempted reservation.
.PP
The \fBdcp_report_key\fR functions allows you to report a result for a
key, provided you have both an authenticated connection and a
reservation on that key.
.PP
The \fBdcp_release_key\fR function allows you to release a single
(string) key, provided you have an authenticated connection.  If the
key argument is NULL, one will be generated by the server and passed back.
The returned report structure represent the data associated with the
key \fIfBafter\fR the release attempt.
.PP
The \fBdcp_release_range\fR function allows you to try to release a
range of numeric keys, provided you have an authenticated connection.
The actual number of valid keys in the range is first returned in
*count.  Then the hook function is called once for each valid key.
The argument passed to the hook represents each valid key's data
\fBafter\fR the attempted release.
.PP
The \fBdcp_close\fR function closes the connection to the DCP server.
.PP
The \fRdcp_error\fR function returns an error message string
corresponding to (but more human-readable than) its argument error code).
.SH AUTHOR
Eric S. Raymond <esr@thyrsus.com>.
.SH SEE ALSO
.IR dcpd (1).
