/* dcpd.h -- declarations for the DCP server */

#define DCP_SERVER_VERSION	"1.0"	/* protocol version # */

#include "dcp.h"		/* stuff common to server and client side */

/* project name to database mapping */
struct dcp_service
{
    char *name;				/* project name */	
    char *desc;				/* description */
    char *file;				/* base name of file */

    int	(*key_ok)(const char *);		/* key validation fn */
    /* validate before recording */
    int (*report_ok)(const char *, struct dcp_entry *, const char *);
    int	(*estimate)(const char *);		/* key time estimate fn */
    void (*key_gen)(char *);			/* generate random key */
    char *(*unpack)(char *);			/* prettyprint report */
    char *(*pack)(char *);			/* compress report */

    time_t	expiry;			/* expiration time in seconds */
};

/* constants for expiry period declarations */
#define MINUTES	60
#define HOURS 	(MINUTES * 60)
#define	DAYS	(HOURS * 24)

/* database layer */
GDBM_FILE entry_open(const char *);
struct dcp_entry *entry_get(GDBM_FILE db, const char *);
int entry_put(GDBM_FILE db, const char *, struct dcp_entry *);
int dcp_getauth(char *, char *, char *);

/* log handling */
void log_init(const char *);
void log_transact(char *format, ...);
void log_close(void);

/* server state machine code */
extern int dcp_server(int, FILE *, struct dcp_service **, FILE *);

/* miscellanea */
void entry_print(struct dcp_service *, struct dcp_entry *, FILE *);
char *passgen(void);
char *nexttok(char *);		/* also used in the test client */

/* dcpd.h ends here */





