/**************************************************************************

NAME
   passgen -- generate a random password

DESCRIPTION
   This program generates ASCII strings that are pronouncible (and thus
easy to remember) of between 5 and 8 characters' length; these are suitable
for use as assigned passwords.
   While there is no intrinsic reason this program shouldn't produce a
string analyzable as an English word, or as a pair of English words
(thus making it theoretically vulnerable to a dictionary crack), the author
has never seen it do so in hundreds of runs.

AUTHOR
   This software is Copyright (C) 1994 by Eric S. Raymond, all rights reserved.

**************************************************************************/
#include <stdio.h>
#include <time.h>

#define MAXLENGTH	8

static char *vowels[] =
{
    "a",
    "e",
    "i",
    "o",
    "u",
};

static char *consonants[] =
{
    "b",
    /* leave out c, pronunciation is ambiguous */
    "d",
    "f",
    "g",
    "h",
    "k",
    "l",
    "m",
    "n",
    "p",
    /* leave out q, it's too confusing as an initial pattern */
    "r",
    "s",
    "t",
    "v",
    /* leave out w, it's too confusing as a medial */
    /* leave out x, it's too confusing as an initial pattern */
    /* leave out y, pronunciation is ambiguous */
    "z",
    "bl",
    "br",
    "ch",
    "fl",
    "fr",
    "gl",
    "gr",
    "kl",
    "kr",
    "pl",
    "pr",
    "sl",
    "sm",
    "sh",
    "st",
    "tr",
};

static char *final[] =
{
    "b",
    /* leave out c, pronunciation is ambiguous */
    "d",
    "f",
    "g",
    "h",
    "k",
    "l",
    "m",
    "n",
    "p",
    /* leave out q, it's too confusing as an initial pattern */
    "r",
    "s",
    "t",
    /* leave out w, it's too confusing as a medial */
    /* leave out x, it's too confusing as an initial pattern */
    /* leave out y, pronunciation is ambiguous */
    "z",
    "ch",
    "nk",
    "nt",
    "sm",
    "sh",
    "st",
};

#define roll(n)		(rand() % (n))
#define length(a)	(sizeof(a) / sizeof(a[0]))
#define pick(w)		w[roll(length(w))]

char *passgen(void)
{
    int	i, jitter, pwdlen;
    static char	pwd[MAXLENGTH + 1];

    pwd[0] = '\0';

    pwdlen = (MAXLENGTH - 3) + roll(3);
    jitter = roll(2);

    for (i = 0; strlen(pwd) < pwdlen; i++)
	if (i % 2 == jitter)
	    (void) strcat(pwd, pick(vowels));
	else if (strlen(pwd) >= pwdlen - 2)
	    (void) strcat(pwd, pick(final));
        else
	    (void) strcat(pwd, pick(consonants));
    return(pwd);
}
