/***************************************************************************

NAME
    rot13.c -- ROT13 test service for DCP

DESCRIPTION
    DCP service functions for ROT13 test service.  This service has
string-valued keys.

AUTHOR
   Eric S. Raymond <esr@thyrsus.com>

***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <gdbm.h>
#include "dcpd.h"

static int key_ok(const char *key)
/* tell if given key is valid for search */
{
    const char	*cp;

    for (cp = key; *cp; cp++)
	if (!isalpha(*cp))
	    return(0);
    return(1);
}

static int record_ok(const char *key, 
		     struct dcp_entry *en, 
		     const char *newdata)
/* validate replacement of property data for given key */
{
    const char	*cp;

    /* return value of this function will increment entry's ntests member */
    for (cp = newdata; *cp; cp++)
	if (!isalpha(*cp))
	    return(0);
    return(1);
}

static int estimate(const char *key)
/* estimate test time in Kflops for given key */
{
    /* this is a stub for testing purposes */
    return(1);
}

static char *prettyprint(char *data)
/* expand property representation into something nice and human-readable */
{
    /* simplest possible transformation */ 
    return(data);
}

static char *compress(char *data)
/* compress property representation into something space-efficient */
{
    /* simplest possible transformation */ 
    return(data);
}

static int roll(int n)
{
    return(((rand() & 0x7FFF) % n));
}

static void key_gen(char *key)
/* generate a good key to search according to server policy */
{
    int i, len;

    for (len = 3 + roll(6); i < len; i++)
	key[i] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"[roll(52)];
}

struct dcp_service rot13 =
{
    "ROT13",				/* project name */
    "Rotate-13 test service",		/* project description */
    "rot13",				/* base filename */

    key_ok,				/* tell if key is valid */
    record_ok,				/* REPORT validation */
    estimate,				/* computation estimate function */
    key_gen,				/* pick random key */
    prettyprint,			/* prettyprint report data */
    compress,				/* compress report data */

    5 * MINUTES,			/* expiry period */
};

/* rot13.c ends here */
